@@ -31,6 +31,7 @@
 
 #include <KLocale>
 #include <KTimeZone>
+#include <KMessageBox>
 #include <KInputDialog>
 #include <KConfigDialog>
 #include <KCalendarSystem>
@@ -40,12 +41,31 @@
 
 K_EXPORT_PLASMA_APPLET(adjustableclock, AdjustableClock)
 
-AdjustableClock::AdjustableClock(QObject *parent, const QVariantList &args) : ClockApplet(parent, args)
+AdjustableClock::AdjustableClock(QObject *parent, const QVariantList &args) : ClockApplet(parent, args),
+    m_controlsTimer(new QTimer(this)),
+    m_toolTipVisible(false),
+    m_predefinedFormat(false)
 {
     KGlobal::locale()->insertCatalog("libplasmaclock");
+    KGlobal::locale()->insertCatalog("timezones4");
     KGlobal::locale()->insertCatalog("adjustableclock");
 
     setHasConfigurationInterface(true);
+
+    m_controlsTimer->setInterval(250);
+    m_controlsTimer->setSingleShot(true);
+
+    m_timeFormatStrings.append(DEFAULT_FORMAT);
+    m_timeFormatStrings.append("<div style=\"text-align:center; margin:5px; white-space:pre;\"><big style=\"font-family:'Nimbus Sans L Condensed';\">%H:%M:%S</big>\n<span style=\"font-size:small; font-family:'Nimbus Sans L';\">%d.%m.%Y</small></div>");
+    m_timeFormatStrings.append("<div style=\"text-align:center; white-space:pre; font-size:25px; margin:5px;\">%H:%M</div>");
+    m_timeFormatStrings.append("<div style=\"text-align:center; white-space:pre; font-size:25px; margin:0 0 5px 5px;\">%H:%M<span style=\"font-size:30px; position:relative; left:-8px; top:4px; z-index:-1; opacity:0.5;\">%S</span></div>");
+    m_timeFormatStrings.append("<div style=\"text-align:center; white-space:pre; font-size:25px; margin:-15px 0 5px 5px; -webkit-box-reflect:below -5px -webkit-gradient(linear, left top, left bottom, from(transparent), color-stop(0.5, transparent), to(white));\">%H:%M<span style=\"font-size:30px; position:relative; left:-8px; top:4px; z-index:-1; opacity:0.5;\">%S</span></div>");
+
+    m_timeFormatNames.append(i18n("Default"));
+    m_timeFormatNames.append(i18n("Flat"));
+    m_timeFormatNames.append(i18n("Simple"));
+    m_timeFormatNames.append(i18n("dbClock"));
+    m_timeFormatNames.append(i18n("dbClock with reflection"));
 }
 
 void AdjustableClock::init()
@@ -54,14 +74,6 @@ void AdjustableClock::init()
 
     ClockApplet::init();
 
-    m_toolTipVisible = false;
-
-    m_placeholdersDialog = NULL;
-
-    m_timer = new QTimer(this);
-    m_timer->setInterval(250);
-    m_timer->setSingleShot(true);
-
     m_clipboardFormats.append("%x");
     m_clipboardFormats.append("%f");
     m_clipboardFormats.append("%H:%M:%S");
@@ -75,7 +87,7 @@ void AdjustableClock::init()
     m_clipboardFormats.append(QString());
     m_clipboardFormats.append("%t");
 
-    m_timeFormat = configuration.readEntry("timeFormat", DEFAULT_FORMAT).replace('\n', "\\n").replace('\'', "\\\'");
+    m_timeFormat = configuration.readEntry("timeFormat", DEFAULT_FORMAT);
     m_toolTipFormat = configuration.readEntry("toolTipFormat", "%Y-%m-%d %H:%M:%S");
     m_fastCopyFormat = configuration.readEntry("fastCopyFormat", "%Y-%m-%d %H:%M:%S");
     m_timeDifference = configuration.readEntry("timeDifference", 0);
@@ -105,7 +117,7 @@ void AdjustableClock::init()
 
     connect(this, SIGNAL(activate()), this, SLOT(copyToClipboard()));
     connect(this, SIGNAL(destroyed()), m_clipboardMenu, SLOT(deleteLater()));
-    connect(m_timer, SIGNAL(timeout()), this, SLOT(updateControls()));
+    connect(m_controlsTimer, SIGNAL(timeout()), this, SLOT(updateControls()));
     connect(m_clipboardMenu, SIGNAL(aboutToShow()), this, SLOT(updateClipboardMenu()));
     connect(m_clipboardMenu, SIGNAL(triggered(QAction*)), this, SLOT(copyToClipboard(QAction*)));
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(updateTheme()));
@@ -116,7 +128,15 @@ void AdjustableClock::dataUpdated(const QString &source, const Plasma::DataEngin
     Q_UNUSED(source)
 
     m_dateTime = QDateTime(data["Date"].toDate(), data["Time"].toTime());
-    m_dateTime = m_dateTime.addSecs(m_timeDifference);
+    m_sunrise = data["Sunrise"].toTime();
+    m_sunset = data["Sunset"].toTime();
+
+    if (m_timeDifference)
+    {
+        m_dateTime = m_dateTime.addSecs(m_timeDifference);
+        m_sunrise = m_sunrise.addSecs(m_timeDifference);
+        m_sunset = m_sunset.addSecs(m_timeDifference);
+    }
 
     setText(formatDateTime(m_dateTime, m_timeFormat));
 
@@ -144,7 +164,7 @@ void AdjustableClock::paintInterface(QPainter *painter, const QStyleOptionGraphi
     m_page.mainFrame()->render(painter);
 }
 
-void AdjustableClock::createConfigurationInterface(KConfigDialog *parent)
+void AdjustableClock::createClockConfigurationInterface(KConfigDialog *parent)
 {
     KConfigGroup configuration = config();
     QStringList timeFormatStrings;
@@ -161,30 +181,74 @@ void AdjustableClock::createConfigurationInterface(KConfigDialog *parent)
     QWidget *advancedConfiguration = new QWidget;
     m_advancedUi.setupUi(advancedConfiguration);
 
-    timeFormatStrings.append(DEFAULT_FORMAT);
-    timeFormatStrings.append("<div style=\"text-align:center; margin:5px; white-space:pre;\"><big style=\"font-family:'Nimbus Sans L Condensed';\">%H:%M:%S</big>\n<span style=\"font-size:small; font-family:'Nimbus Sans L';\">%d.%m.%Y</small></div>");
-    timeFormatStrings.append("<div style=\"text-align:center; white-space:pre; font-size:25px; margin:5px;\">%H:%M</div>");
-    timeFormatStrings.append("<div style=\"text-align:center; white-space:pre; font-size:25px; margin:0 0 5px 5px;\">%H:%M<span style=\"font-size:30px; position:relative; left:-8px; top:4px; z-index:-1; opacity:0.5;\">%S</span></div>");
-    timeFormatStrings.append("<div style=\"text-align:center; white-space:pre; font-size:25px; margin:-15px 0 5px 5px; -webkit-box-reflect:below -5px -webkit-gradient(linear, left top, left bottom, from(transparent), color-stop(0.5, transparent), to(white));\">%H:%M<span style=\"font-size:30px; position:relative; left:-8px; top:4px; z-index:-1; opacity:0.5;\">%S</span></div>");
+    KMenu *placeholdersMenu = new KMenu(m_appearanceUi.placeholdersButton);
 
-    timeFormatNames.append(i18n("Default"));
-    timeFormatNames.append(i18n("Flat"));
-    timeFormatNames.append(i18n("Simple"));
-    timeFormatNames.append(i18n("dbClock"));
-    timeFormatNames.append(i18n("dbClock with reflection"));
+    QMap<QChar, QString> placeholders;
 
-    timeFormatStrings = configuration.readEntry("timeFormatStrings", timeFormatStrings);
+    placeholders['H'] = i18n("The hour in 24h format with two digits");
+    placeholders['k'] = i18n("The hour in 24h format");
+    placeholders['I'] = i18n("The hour in 12h format with two digits");
+    placeholders['l'] = i18n("The hour in 12h format");
+    placeholders['M'] = i18n("The minute number with two digits");
+    placeholders['S'] = i18n("The seconds number with two digits");
+    placeholders['s'] = i18n("The seconds number");
+    placeholders['p'] = i18n("The pm or am string");
+    placeholders['Y'] = i18n("The whole year");
+    placeholders['y'] = i18n("The lower two digits of the year");
+    placeholders['n'] = i18n("The month number");
+    placeholders['m'] = i18n("The month number with two digits");
+    placeholders['e'] = i18n("The day of the month number");
+    placeholders['d'] = i18n("The day of the month number with two digits");
+    placeholders['b'] = i18n("The short form of the month name");
+    placeholders['B'] = i18n("The long form of the month name");
+    placeholders['a'] = i18n("The short form of the weekday name");
+    placeholders['A'] = i18n("The long form of the weekday name");
+    placeholders['g'] = i18n("The timezone city string");
+    placeholders['z'] = i18n("The timezone offset to UTC");
+    placeholders['Z'] = i18n("The timezone abbreviation string");
+    placeholders['j'] = i18n("The day of year number");
+    placeholders['w'] = i18n("The weekday number");
+    placeholders['W'] = i18n("The week number");
+    placeholders['c'] = i18n("The default short date and time format");
+    placeholders['C'] = i18n("The default long date and time format");
+    placeholders['x'] = i18n("The default long date format");
+    placeholders['X'] = i18n("The default long time format");
+    placeholders['f'] = i18n("The default short date format");
+    placeholders['F'] = i18n("The default short time format");
+    placeholders['t'] = i18n("The UNIX timestamp");
+    placeholders['o'] = i18n("The sunrise time");
+    placeholders['O'] = i18n("The sunset time");
 
-    timeFormatNames = configuration.readEntry("timeFormatNames", timeFormatNames);
+    QMap<QChar, QString>::const_iterator iterator = placeholders.constBegin();
+
+    while (iterator != placeholders.constEnd())
+    {
+        QAction *action = placeholdersMenu->addAction(QString("%1 (%%2)\t%3").arg(iterator.value()).arg(iterator.key()).arg(formatDateTime(m_dateTime, (QString('%').append(iterator.key())))));
+        action->setData(iterator.key());
+
+        ++iterator;
+    }
+
+    timeFormatStrings = m_timeFormatStrings;
+    timeFormatStrings.append(configuration.readEntry("timeFormatStrings", QStringList()));
+
+    timeFormatNames = m_timeFormatNames;
+    timeFormatNames.append(configuration.readEntry("timeFormatNames", QStringList()));
 
     for (int i = 0; i < timeFormatStrings.count(); ++i)
     {
+        if (i == m_timeFormatStrings.count())
+        {
+            m_appearanceUi.timeFormatComboBox->insertSeparator(i);
+        }
+
         m_appearanceUi.timeFormatComboBox->addItem(timeFormatNames.at(i), timeFormatStrings.at(i));
     }
 
     m_appearanceUi.addButton->setIcon(KIcon("list-add"));
     m_appearanceUi.removeButton->setIcon(KIcon("list-remove"));
-    m_appearanceUi.placeholdersButton->setIcon(KIcon("help-hint"));
+    m_appearanceUi.placeholdersButton->setIcon(KIcon("chronometer"));
+    m_appearanceUi.placeholdersButton->setMenu(placeholdersMenu);
     m_appearanceUi.boldButton->setIcon(KIcon("format-text-bold"));
     m_appearanceUi.italicButton->setIcon(KIcon("format-text-italic"));
     m_appearanceUi.underlineButton->setIcon(KIcon("format-text-underline"));
@@ -227,13 +291,13 @@ void AdjustableClock::createConfigurationInterface(KConfigDialog *parent)
     m_clipboardUi.moveUpButton->setIcon(KIcon("arrow-up"));
     m_clipboardUi.moveDownButton->setIcon(KIcon("arrow-down"));
 
-    foreach (const QString &string, m_clipboardFormats)
+    for (int i = 0; i < m_clipboardFormats.count(); ++i)
     {
         row = m_clipboardUi.clipboardActionsTable->rowCount();
         m_clipboardUi.clipboardActionsTable->insertRow(row);
-        m_clipboardUi.clipboardActionsTable->setItem(row, 0, new QTableWidgetItem(string));
+        m_clipboardUi.clipboardActionsTable->setItem(row, 0, new QTableWidgetItem(m_clipboardFormats.at(i)));
 
-        preview = formatDateTime(m_dateTime, string);
+        preview = formatDateTime(m_dateTime, m_clipboardFormats.at(i));
 
         QTableWidgetItem *item = new QTableWidgetItem(preview);
         item->setFlags(0);
@@ -247,17 +311,14 @@ void AdjustableClock::createConfigurationInterface(KConfigDialog *parent)
     itemSelectionChanged();
 
     parent->addPage(appearanceConfiguration, i18n("Appearance"), "preferences-desktop-theme");
-
-    ClockApplet::createConfigurationInterface(parent);
-
     parent->addPage(clipboardActions, i18n("Clipboard actions"), "edit-copy");
     parent->addPage(advancedConfiguration, i18n("Advanced"), "configure");
 
     connect(parent, SIGNAL(finished()), this, SLOT(resetTimeFormat()));
+    connect(placeholdersMenu, SIGNAL(triggered(QAction*)), this, SLOT(insertPlaceholder(QAction*)));
     connect(m_appearanceUi.timeFormatComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(loadItem(int)));
     connect(m_appearanceUi.addButton, SIGNAL(clicked()), this, SLOT(addItem()));
     connect(m_appearanceUi.removeButton, SIGNAL(clicked()), this, SLOT(removeItem()));
-    connect(m_appearanceUi.placeholdersButton, SIGNAL(clicked()), this, SLOT(showPlaceholdersDialog()));
     connect(m_appearanceUi.webView->page(), SIGNAL(selectionChanged()), this, SLOT(selectionChanged()));
     connect(m_appearanceUi.webView->page(), SIGNAL(contentsChanged()), this, SLOT(contentsChanged()));
     connect(m_appearanceUi.timeFormat, SIGNAL(textChanged()), this, SLOT(formatChanged()));
@@ -298,10 +359,15 @@ void AdjustableClock::configAccepted()
     QStringList timeFormatStrings;
     QStringList timeFormatNames;
 
-    dataEngine("time")->disconnectSource(currentTimezone(), this);
+    dataEngine("time")->disconnectSource((currentTimezone() + "|Solar"), this);
 
     for (int i = 0; i < m_appearanceUi.timeFormatComboBox->count(); ++i)
     {
+        if (m_timeFormatNames.contains(m_appearanceUi.timeFormatComboBox->itemText(i)) || m_appearanceUi.timeFormatComboBox->itemText(i).isEmpty())
+        {
+            continue;
+        }
+
         timeFormatStrings.append(m_appearanceUi.timeFormatComboBox->itemData(i).toString());
 
         timeFormatNames.append(m_appearanceUi.timeFormatComboBox->itemText(i));
@@ -336,13 +402,11 @@ void AdjustableClock::configAccepted()
 
     m_clipboardAction->setVisible(!m_clipboardFormats.isEmpty());
 
-    m_timeFormat.replace('\n', "\\n").replace('\'', "\\\'");
-
     ClockApplet::configAccepted();
 
     connectSource(currentTimezone());
 
-    m_timer->stop();
+    m_controlsTimer->stop();
 
     emit configNeedsSaving();
 }
@@ -352,7 +416,7 @@ void AdjustableClock::connectSource(const QString &timezone)
     QRegExp formatWithSeconds = QRegExp("%(S|c|C|t|F|X)");
     const bool alignToSeconds = (m_timeFormat.contains(formatWithSeconds) || m_toolTipFormat.contains(formatWithSeconds));
 
-    dataEngine("time")->connectSource(timezone, this, (alignToSeconds?1000:60000), (alignToSeconds?Plasma::NoAlignment:Plasma::AlignToMinute));
+    dataEngine("time")->connectSource((timezone + "|Solar"), this, (alignToSeconds?1000:60000), (alignToSeconds?Plasma::NoAlignment:Plasma::AlignToMinute));
 
     m_timeZoneAbbreviation = KSystemTimeZones::zone(timezone).abbreviation(QDateTime::currentDateTime().toUTC());
 
@@ -381,7 +445,7 @@ void AdjustableClock::connectSource(const QString &timezone)
         m_timeZoneOffset.append(QString::number(minutes));
     }
 
-    m_timeZoneOffset = (((seconds >= 0)?"+":"-") + m_timeZoneOffset);
+    m_timeZoneOffset = (QChar((seconds >= 0)?'+':'-') + m_timeZoneOffset);
 
     updateSize();
 }
@@ -401,34 +465,74 @@ void AdjustableClock::copyToClipboard()
     QApplication::clipboard()->setText(formatDateTime(currentDateTime(), m_fastCopyFormat));
 }
 
-void AdjustableClock::showPlaceholdersDialog()
+void AdjustableClock::insertPlaceholder(QAction *action)
 {
-    if (!m_placeholdersDialog)
+    QString placeholder = QString('%').append(action->data().toChar());
+
+    if (m_appearanceUi.tabWidget->currentIndex() > 0)
     {
-        m_placeholdersDialog = new KDialog;
-        m_placeholdersDialog->setButtons(KDialog::Close);
-        m_placeholdersDialog->setWindowTitle(i18n("Placeholders"));
-
-        QWidget *placeholders = new QWidget;
-        m_placeholdersUi.setupUi(placeholders);
-
-        m_placeholdersDialog->setMainWidget(placeholders);
-
-        connect(this, SIGNAL(destroyed()), m_placeholdersDialog, SLOT(deleteLater()));
+        m_appearanceUi.timeFormat->insertPlainText(placeholder);
+    }
+    else
+    {
+        m_appearanceUi.webView->page()->mainFrame()->evaluateJavaScript("document.execCommand('inserthtml', false, '" + placeholder + "')");
     }
-
-    m_placeholdersDialog->show();
 }
 
 void AdjustableClock::loadItem(int index)
 {
+    m_predefinedFormat = false;
+
     m_appearanceUi.timeFormat->setPlainText(m_appearanceUi.timeFormatComboBox->itemData(index).toString());
+
+    m_predefinedFormat = m_timeFormatNames.contains(m_appearanceUi.timeFormatComboBox->itemText(index));
+
+    m_appearanceUi.removeButton->setEnabled(!m_predefinedFormat);
 }
 
-void AdjustableClock::addItem()
+void AdjustableClock::addItem(bool automatically)
 {
-    m_appearanceUi.timeFormatComboBox->insertItem(m_appearanceUi.timeFormatComboBox->currentIndex(), KInputDialog::getText(i18n("Add new format"), i18n("Format name:")), m_appearanceUi.timeFormat->toPlainText());
-    m_appearanceUi.timeFormatComboBox->setCurrentIndex(m_appearanceUi.timeFormatComboBox->currentIndex() - 1);
+    QString formatName = m_appearanceUi.timeFormatComboBox->itemText(m_appearanceUi.timeFormatComboBox->currentIndex());
+
+    if (automatically)
+    {
+        int i = 2;
+
+        while (m_appearanceUi.timeFormatComboBox->findText(QString("%1 %2").arg(formatName).arg(i)) >= 0)
+        {
+            ++i;
+        }
+
+        formatName = QString("%1 %2").arg(formatName).arg(i);
+    }
+    else
+    {
+        formatName = KInputDialog::getText(i18n("Add new format"), i18n("Format name:"), formatName);
+    }
+
+    if (m_appearanceUi.timeFormatComboBox->findText(formatName) >= 0)
+    {
+        KMessageBox::error(m_appearanceUi.timeFormatComboBox, i18n("Format with this name already exists!"));
+    }
+    else if (!formatName.isEmpty())
+    {
+        int index = (m_appearanceUi.timeFormatComboBox->currentIndex() + 1);
+
+        if (index <= m_timeFormatNames.count())
+        {
+            index = m_appearanceUi.timeFormatComboBox->count();
+        }
+
+        m_predefinedFormat = false;
+
+        disconnect(m_appearanceUi.timeFormatComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(loadItem(int)));
+
+        m_appearanceUi.timeFormatComboBox->insertItem(index, formatName, m_appearanceUi.timeFormat->toPlainText());
+        m_appearanceUi.timeFormatComboBox->setCurrentIndex(index);
+        m_appearanceUi.removeButton->setEnabled(true);
+
+        connect(m_appearanceUi.timeFormatComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(loadItem(int)));
+    }
 }
 
 void AdjustableClock::removeItem()
@@ -445,11 +549,11 @@ void AdjustableClock::removeItem()
 void AdjustableClock::updateControls()
 {
     m_appearanceUi.webView->page()->mainFrame()->evaluateJavaScript("boldButton.setChecked(document.queryCommandState('bold'));"
-                                                         "italicButton.setChecked(document.queryCommandState('italic'));"
-                                                         "underlineButton.setChecked(document.queryCommandState('underline'));"
-                                                         "designModeEditor.setColor(document.queryCommandValue('forecolor'));"
-                                                         "designModeEditor.setFontSize((document.queryCommandValue('fontsize') == '9px')?9:document.queryCommandValue('fontsize').substr(0, 2));"
-                                                         "designModeEditor.setFontFamily(document.queryCommandValue('fontname'))");
+        "italicButton.setChecked(document.queryCommandState('italic'));"
+        "underlineButton.setChecked(document.queryCommandState('underline'));"
+        "designModeEditor.setColor(document.queryCommandValue('forecolor'));"
+        "designModeEditor.setFontSize((document.queryCommandValue('fontsize') == '9px')?9:document.queryCommandValue('fontsize').substr(0, 2));"
+        "designModeEditor.setFontFamily(document.queryCommandValue('fontname'))");
 }
 
 void AdjustableClock::toggleState()
@@ -527,29 +631,54 @@ void AdjustableClock::contentsChanged()
 
     disconnect(m_appearanceUi.timeFormat, SIGNAL(textChanged()), this, SLOT(formatChanged()));
 
-    m_appearanceUi.timeFormatComboBox->setItemData(m_appearanceUi.timeFormatComboBox->currentIndex(), text);
+    QString toolTip = formatDateTime(m_dateTime, text, false);
+
+    m_appearanceUi.timeFormat->setToolTip(toolTip);
+    m_appearanceUi.webView->setToolTip(toolTip);
     m_appearanceUi.timeFormat->setPlainText(text);
 
+    if (m_predefinedFormat)
+    {
+        addItem(true);
+    }
+
+    m_appearanceUi.timeFormatComboBox->setItemData(m_appearanceUi.timeFormatComboBox->currentIndex(), text);
+
     connect(m_appearanceUi.timeFormat, SIGNAL(textChanged()), this, SLOT(formatChanged()));
 }
 
 void AdjustableClock::formatChanged()
 {
+    if (m_appearanceUi.timeFormat->toPlainText() == m_appearanceUi.webView->page()->mainFrame()->toHtml())
+    {
+        return;
+    }
+
     disconnect(m_appearanceUi.webView->page(), SIGNAL(contentsChanged()));
 
-    m_appearanceUi.timeFormatComboBox->setItemData(m_appearanceUi.timeFormatComboBox->currentIndex(), m_appearanceUi.timeFormat->toPlainText());
+    QString toolTip = formatDateTime(m_dateTime, m_appearanceUi.timeFormat->toPlainText(), false);
+
+    m_appearanceUi.timeFormat->setToolTip(toolTip);
+    m_appearanceUi.webView->setToolTip(toolTip);
     m_appearanceUi.webView->page()->mainFrame()->setHtml(m_appearanceUi.timeFormat->toPlainText());
     m_appearanceUi.webView->page()->mainFrame()->addToJavaScriptWindowObject("boldButton", m_appearanceUi.boldButton);
     m_appearanceUi.webView->page()->mainFrame()->addToJavaScriptWindowObject("italicButton", m_appearanceUi.italicButton);
     m_appearanceUi.webView->page()->mainFrame()->addToJavaScriptWindowObject("underlineButton", m_appearanceUi.underlineButton);
     m_appearanceUi.webView->page()->mainFrame()->addToJavaScriptWindowObject("designModeEditor", this);
 
+    if (m_predefinedFormat)
+    {
+        addItem(true);
+    }
+
+    m_appearanceUi.timeFormatComboBox->setItemData(m_appearanceUi.timeFormatComboBox->currentIndex(), m_appearanceUi.timeFormat->toPlainText());
+
     connect(m_appearanceUi.webView->page(), SIGNAL(contentsChanged()), this, SLOT(contentsChanged()));
 }
 
 void AdjustableClock::selectionChanged()
 {
-    m_timer->start();
+    m_controlsTimer->start();
 }
 
 void AdjustableClock::itemSelectionChanged()
@@ -665,7 +794,7 @@ void AdjustableClock::updateClipboardMenu()
 
 void AdjustableClock::changeEngineTimezone(const QString &oldTimezone, const QString &newTimezone)
 {
-    dataEngine("time")->disconnectSource(oldTimezone, this);
+    dataEngine("time")->disconnectSource((oldTimezone + "|Solar"), this);
 
     connectSource(newTimezone);
 }
@@ -677,7 +806,7 @@ void AdjustableClock::updateToolTipContent()
     if (!m_toolTipFormat.isEmpty())
     {
         toolTipData.setImage(KIcon("chronometer").pixmap(IconSize(KIconLoader::Desktop)));
-        toolTipData.setMainText(formatDateTime(m_dateTime, m_toolTipFormat));
+        toolTipData.setMainText(formatDateTime(m_dateTime, m_toolTipFormat, false));
         toolTipData.setAutohide(false);
     }
 
@@ -701,7 +830,7 @@ void AdjustableClock::updateSize()
 
             placeholder = m_timeFormat.at(index).unicode();
 
-            longest = QString();
+            longest.clear();
 
             switch (placeholder)
             {
@@ -764,6 +893,12 @@ void AdjustableClock::updateSize()
                 case 'x':
                     string.append(KGlobal::locale()->formatDate(m_dateTime.date(), KLocale::LongDate));
                 break;
+                case 'o':
+                    string.append(KGlobal::locale()->formatTime(m_sunrise, false));
+                break;
+                case 'O':
+                    string.append(KGlobal::locale()->formatTime(m_sunset, false));
+                break;
                 case 'F':
                     string.append(KGlobal::locale()->formatTime(m_dateTime.time(), false));
                 break;
@@ -871,7 +1006,7 @@ QDateTime AdjustableClock::currentDateTime()
     return dateTime;
 }
 
-QString AdjustableClock::formatDateTime(const QDateTime dateTime, QString format)
+QString AdjustableClock::formatDateTime(const QDateTime dateTime, QString format, bool clean)
 {
     if (format.isEmpty())
     {
@@ -899,12 +1034,12 @@ QString AdjustableClock::formatDateTime(const QDateTime dateTime, QString format
                     string.append(KGlobal::locale()->calendar()->monthName(KGlobal::locale()->calendar()->month(dateTime.date()), KGlobal::locale()->calendar()->year(dateTime.date()), ((KGlobal::locale()->nounDeclension() && KGlobal::locale()->dateMonthNamePossessive())?KCalendarSystem::ShortNamePossessive:KCalendarSystem::ShortName)));
                 break;
                 case 'B': // month, long form
-                    string.append(KGlobal::locale()->calendar()->monthName(KGlobal::locale()->calendar()->month(dateTime.date()), KGlobal::locale()->calendar()->year(dateTime.date()), ((KGlobal::locale()->nounDeclension() && KGlobal::locale()->dateMonthNamePossessive())?KCalendarSystem::LongNamePossessive:KCalendarSystem::LongName)));
+                    string.append(KGlobal::locale()->calendar()->monthName(KGlobal::locale()->calendar()->month(dateTime.date()), KGlobal::locale()->calendar()->year(dateTime.date()), KCalendarSystem::LongName));
                 break;
-                case 'c': // date and time representation, short
+                case 'c': // date and time format, short
                     string.append(KGlobal::locale()->formatDateTime(dateTime, KLocale::LongDate));
                 break;
-                case 'C': // date and time representation, long
+                case 'C': // date and time format, long
                     string.append(KGlobal::locale()->formatDateTime(dateTime, KLocale::ShortDate));
                 break;
                 case 'd': // day of the month, two digits
@@ -913,10 +1048,10 @@ QString AdjustableClock::formatDateTime(const QDateTime dateTime, QString format
                 case 'e': // day of the month, one digit
                     string.append(KGlobal::locale()->calendar()->dayString(dateTime.date(), KCalendarSystem::ShortFormat));
                 break;
-                case 'f': // date representation, short
+                case 'f': // date format, short
                     string.append(KGlobal::locale()->formatDate(dateTime.date(), KLocale::ShortDate));
                 break;
-                case 'F': // time representation, short
+                case 'F': // time format, short
                     string.append(KGlobal::locale()->formatTime(dateTime.time(), false));
                 break;
                 case 'g': // timezone city
@@ -961,9 +1096,18 @@ QString AdjustableClock::formatDateTime(const QDateTime dateTime, QString format
                 case 'n': // month, one digit
                     string.append(KGlobal::locale()->calendar()->monthString(dateTime.date(), KCalendarSystem::ShortFormat));
                 break;
+                case 'o': // sunrise time
+                    string.append(KGlobal::locale()->formatTime(m_sunrise, false));
+                break;
+                case 'O': // sunset time
+                    string.append(KGlobal::locale()->formatTime(m_sunset, false));
+                break;
                 case 'p': // pm or am
                     string.append((dateTime.time().hour() >= 12)?i18n("pm"):i18n("am"));
                 break;
+                case 's': // second, one digit
+                    string.append(QString::number(dateTime.time().second()));
+                break;
                 case 'S': // second, two digits
                     if (dateTime.time().second() < 10)
                     {
@@ -976,16 +1120,16 @@ QString AdjustableClock::formatDateTime(const QDateTime dateTime, QString format
                     string.append(QString::number(dateTime.toTime_t()));
                 break;
                 case 'w': // day of week
-                    string.append(KGlobal::locale()->calendar()->dayOfWeek(dateTime.date()));
+                    string.append(QString::number(KGlobal::locale()->calendar()->dayOfWeek(dateTime.date())));
                 break;
                 case 'W': // week number
                 case 'U':
                     string.append(QString::number(KGlobal::locale()->calendar()->weekNumber(dateTime.date())));
                 break;
-                case 'x': // date representation, long
+                case 'x': // date format, long
                     string.append(KGlobal::locale()->formatDate(dateTime.date(), KLocale::LongDate));
                 break;
-                case 'X': // time representation, long
+                case 'X': // time format, long
                     string.append(KGlobal::locale()->formatTime(dateTime.time(), true));
                 break;
                 case 'Y': // year, four digits
@@ -1005,6 +1149,14 @@ QString AdjustableClock::formatDateTime(const QDateTime dateTime, QString format
                 break;
             }
         }
+        else if (clean && format.at(index) == '\n')
+        {
+            string.append("\\n");
+        }
+        else if (clean && format.at(index) == '\'')
+        {
+            string.append("\\'");
+        }
         else
         {
             string.append(format.at(index));
@@ -1024,9 +1176,9 @@ QList<QAction*> AdjustableClock::clipboardActions()
     QList<QAction*> actions;
     QDateTime dateTime = currentDateTime();
 
-    foreach (const QString &string, m_clipboardFormats)
+    for (int i = 0; i < m_clipboardFormats.count(); ++i)
     {
-        if (string.isEmpty())
+        if (m_clipboardFormats.at(i).isEmpty())
         {
             QAction *action = new QAction(NULL);
             action->setSeparator(true);
@@ -1035,7 +1187,7 @@ QList<QAction*> AdjustableClock::clipboardActions()
         }
         else
         {
-            actions.append(new QAction(formatDateTime(dateTime, string), NULL));
+            actions.append(new QAction(formatDateTime(dateTime, m_clipboardFormats.at(i)), NULL));
         }
     }
 
@@ -1044,8 +1196,16 @@ QList<QAction*> AdjustableClock::clipboardActions()
 
 QList<QAction*> AdjustableClock::contextualActions()
 {
-    QList<QAction*> actions;
-    actions.append(m_clipboardAction);
+    QList<QAction*> actions = ClockApplet::contextualActions();
+
+    for (int i = 0; i < actions.count(); ++i)
+    {
+        if (actions.at(i)->text() == i18n("C&opy to Clipboard"))
+        {
+            actions.removeAt(i);
+            actions.insert(i, m_clipboardAction);
+        }
+    }
 
     return actions;
 }
